package org.kldp.zeta.interpret4j;

import java.io.BufferedReader;
import java.io.FileReader;

import org.kldp.zeta.interpret4j.language.BeanShellSupport;
import org.kldp.zeta.interpret4j.language.CookXMLSupport;
import org.kldp.zeta.interpret4j.language.GroovySupport;
import org.kldp.zeta.interpret4j.language.ILanguageSupport;
import org.kldp.zeta.interpret4j.language.JavascriptSupport;
import org.kldp.zeta.interpret4j.language.LispSupport;
import org.kldp.zeta.interpret4j.language.PythonSupport;
import org.kldp.zeta.interpret4j.language.RubySupport;
import org.kldp.zeta.interpret4j.language.TclSupport;

/**
 * @author ni2
 * @version 0.1, 10/04/06
 * This code licensed under the terms of the LGPL.  See doc/LICENSE.
 */
public class Interpreter implements IInterpreter{
	private static final Interpreter instance = new Interpreter();
	
	public enum Type{
		BEANSHELL("bsh", BeanShellSupport.class),
		GROOVY("groovy", GroovySupport.class),
		JAVASCRIPT("js", JavascriptSupport.class),
		LISP(new String[]{"lsp","lisp"}, LispSupport.class),
		PYTHON(new String[]{"py","jy"}, PythonSupport.class),
		RUBY("rb", RubySupport.class),
		TCL("tcl", TclSupport.class),
		XML_COOKXML("xml", CookXMLSupport.class);
		
		private ILanguageSupport interpreter;
		public final Class interpreterClass;
		public final String[] supportedExtensions;
		Type(String[] exts, Class cls){
			this.supportedExtensions = exts;
			this.interpreterClass = cls;
		}
		Type(String ext, Class cls){
			supportedExtensions = new String[1];
			supportedExtensions[0] = ext;
			this.interpreterClass = cls;
		}
		public ILanguageSupport getInterpreter(){
			if(interpreter == null){
				try {
					interpreter = (ILanguageSupport) interpreterClass.newInstance();
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
			return interpreter;
		}
	}
	
	public static Interpreter instance(){
		return instance;
	}
	
	public Object eval(String source, Type type) throws Exception{
		return type.getInterpreter().eval(source);
	}
	
	public Object interpretFromFile(String fileName) throws Exception{
		Type type = null;
		String lowerName = fileName.toLowerCase();
		
		outer:
		for(Type t : Type.values()){
			for(String ext : t.supportedExtensions)
				if(lowerName.endsWith(ext)){
					type = t;
					break outer;
				}
		}
		return interpretFromFile(fileName, type);
	}
	
	public Object interpretFromFile(String fileName, Type type) throws Exception{
		return type.getInterpreter().interpretFromFile(fileName);
	}
	
	/*
	 * LanguageSupport 클래스들에서 사용하기 위한 소스 리더 메서드.
	 * '내장 인터프리터 레퍼런스'를 지향하는 interpret4j의 특성상,
	 * 가급적 각각의 라이브러리 내에 같은 기능의 구현이 없을 때에만 사용한다. 
	 */
	public static String loadSource(String fileName) throws Exception{
		BufferedReader br = new BufferedReader(new FileReader(fileName));
		StringBuilder sb = new StringBuilder();
		String tmp;
		while((tmp = br.readLine()) != null)
			sb.append(tmp);
		
		return sb.toString();
	}
}
