package ni2.likewind.io;

import java.io.File;
import java.io.FilenameFilter;

import ni2.likewind.image.ImageLoader;

/**
 * 주어진 파일과 같은 디렉터리에 있는 모든 이미지 파일을 수집한다.
 * 하위 디렉터리는 제외.
 * 
 * @author ni2 (netisinfinite@gmail.com)
 * @version 0.1 11/07/06
 * This code licensed under the terms of the GPL.  See doc/LICENSE.
 */
public class DirectoryCrawler {
	public FilenameFilter nameFilter = new FilenameFilter(){
		public boolean accept(File dir, String name) {
			name = name.toLowerCase();
			for(String ext : ImageLoader.SUPPORTED_FILE_TYPES){
				if(name.endsWith('.' + ext)){
					return true;
				}
			}
			return false;
		}
	};
	
	private File[] files;
	private int selectedIndex;
	
	public void crawl(String fileName){
		if(files != null){
			for(File f : files){
				if(fileName.equals(f.getPath()))
					return;
			}
		}
		File file = new File(fileName);
		File[] fileNames = new File(file.getParent()).listFiles(nameFilter);
		if(fileNames != null){
			files = fileNames;
			for(int i=0; i<fileNames.length; i++){
				if(file.equals(fileNames[i]))
					selectedIndex = i;
			}
		}
	}
	
	public int count(){
		return (files != null) ? files.length : -1;
	}
	
	public int index(){
		return (files != null) ? selectedIndex : -1;
	}
	
	public File next(){
		File file = null;
		if(files != null){
			int nextIndex = selectedIndex + 1;
			if(nextIndex < files.length){
				file = files[nextIndex];
				selectedIndex = nextIndex;
			}
		}
		return file;
	}
	
	public File previous(){
		File file = null;
		if(files != null && selectedIndex > 0){
			file = files[--selectedIndex];
		}
		return file;
	}
}
