package ni2.likewind.ui;

import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.dnd.FileTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

/**
 * 이미지를 그리는 캔버스.
 * 
 * @author ni2 (netisinfinite@gmail.com)
 * @version 0.1 11/14/06
 * This code licensed under the terms of the GPL.  See doc/LICENSE.
 */
public class ImageCanvas extends Canvas{
	private ImageCanvasData data;
	private GridData layoutData;
	private int viewportX, viewportY;
	private int width, height;
	
	private boolean fitToScreen;
	
	public ImageCanvas(Composite parent, int style){
		super(parent, style | SWT.DOUBLE_BUFFERED);
		data = new ImageCanvasData();
		CanvasListener listener = new CanvasListener();
		addMouseMoveListener(listener);
		addListener(SWT.Resize, listener);
		addListener(SWT.MouseWheel, listener);
		addListener(SWT.MouseDown, listener);
		addListener(SWT.MouseUp, listener);
		addPaintListener(listener);
		
		DropTarget target = new DropTarget(this, DND.DROP_MOVE);
		target.setTransfer(new Transfer[]{FileTransfer.getInstance()});
		target.addDropListener(listener);
	}
	
	public void setLayoutData(GridData data){
		super.setLayoutData(data);
		layoutData = data;
	}
	
	public boolean getCanvasImageSizeMatched(){
		if(viewportX >= 0 && viewportY >= 0
			&& viewportX + data.imageWidthMag == width
			&& viewportY + data.imageHeightMag == height)
			return true;
		return false;
	}
	
	private void drawImage(GC gc){
		if (data.image != null){
			if(fitToScreen){
				gc.drawImage(
						data.image,
						0,
						0,
						data.imageWidth,
						data.imageHeight,
						0,
						0,
						width,
						height);
			}else{
				/*
				 * destX, destY(0,0)으로부터 그려나갈 그림의 부분이다.
				 * 예컨데 현재 이미지의 위치가 화면 밖의 -10, -10이라면 이 값은 10, 10이 된다.
				 */
				int srcX = viewportX >= 0 ? 0 : -viewportX;
				int srcY = viewportY >= 0 ? 0 : -viewportY;
				/*
				 * destX, destY는 viewport 내에서  그리기 시작할 지점이다.
				 * viewport 시작 지점이 음수일 경우 0으로 고정된다.
				 */
				int destX = viewportX <= 0 ? 0 : viewportX;
				int destY = viewportY <= 0 ? 0 : viewportY;
				/*
				 * destWidth, destHeight은 viewport 밖의 부분을 그리지 않음으로서 리소스를
				 * 낭비하는 것을 막기 위해 지정한다. 이를테면 캔버스의 크기가 200이고 그림의 크기가
				 * 800이면, dest는 200으로 지정되어 필요한 만큼만 그리도록 한다.
				 * TODO 연산을 좀 더 단순하게 할 방법은 없을까?
				 */
				int srcWidth = data.imageWidth >= width + srcX ?
						width : data.imageWidth - srcX;
				int srcHeight = data.imageHeight >= height + srcY ?
						height : data.imageHeight - srcY;
				int destWidth = (int)(srcWidth * data.magnification);
				int destHeight = (int)(srcHeight * data.magnification);
				int tmp;
				if(destX + destWidth > width){
					tmp = (destX + destWidth) - width;
					srcWidth -= (int)(tmp / data.magnification);
					destWidth -= tmp;
				}
				if(destY + destHeight > height){
					tmp = (destY + destHeight) - height;
					srcHeight -= (int)(tmp / data.magnification);
					destHeight -= tmp;
				}
				//System.out.println("===========");
				//System.out.println("magnification " + magnification);
				//System.out.println("viewportX " + viewportX + ", viewportY " + viewportY);
				//System.out.println("srcX " + srcX + ", srcY " + srcY);
				//System.out.println("width " + width + ", height " + height);
				//System.out.println("destX " + destX + ", destY " + destY);
				//System.out.println("srcWidth " + srcWidth + ", srcHeight " + srcHeight);
				//System.out.println("destWidth " + destWidth + ", destHeight " + destHeight);
	
				gc.drawImage(
						data.image,
						srcX,
						srcY,
						srcWidth,
						srcHeight,
						destX,
						destY,
						destWidth,
						destHeight);
			}
		}
	}
	
	class CanvasListener implements Listener, DropTargetListener, MouseMoveListener, PaintListener{
		private int savedX, savedY;
		private boolean windowMove, scrollImage;
		private Rectangle windowBounds;
		
		public void mouseDown(Event e){
			if(e.button == 1){
				if(checkImage()){
					savedX = e.x;
					savedY = e.y; 
					if((! Wind.instance.isMaxmized()) && getCanvasImageSizeMatched()){
						/*
						 * Control.getBounds()는 실행시마다 새로 계산을 실행해 Rect 객체를 만들어낸다.
						 * 이를 최소화하기 위해 클릭 시점에서 객체를 받아 저장한다.
						 */
						windowBounds = Wind.instance.getBounds();
						windowMove = true;
					}else{
						scrollImage = true;
					}
					setCursor(Wind.CURSOR_GRAB);
				}
			}else if(e.button == 2){
				Wind.instance.maxmize();
			}
		}
		
		public void mouseUp(Event e){
			windowMove = false;
			scrollImage = false;
			setCursor(null);
		}
		
		public void mouseMove(MouseEvent e){
			if(scrollImage){
				/*
				 * LikeWind는 사용자가 그림을 자유로이 이동시킬 수 있기 때문에 문제가 발생한다.
				 * Canvas에서, 기본적으로 음수 좌표로부터의 그리기(이를테면 -100 지점부터
				 * 200까지)는 허용된다. 하지만 viewport에서 완전히 넘어가는 지점, 즉
				 * -100 지점부터 -1까지 그린다고 하면 InvalidArgument 예외가 발생한다.
				 * 아래의 코드는, 그림이 화면 밖으로 완전히 넘어가는 것을 막아준다.
				 */
				if(viewportX - (savedX - e.x) < width
						&& viewportX - (savedX - e.x) > -data.imageWidth)
					viewportX -= (savedX - e.x);
				if(viewportY - (savedY - e.y) < height
						&& viewportY - (savedY - e.y) > -data.imageHeight)
					viewportY -= (savedY - e.y);
				//FIXME 아래 if(windowMove)를 참고하여 아래 두 줄을 없앨것
				savedX = e.x;
				savedY = e.y;
				redraw();
			}else if(windowMove){
				windowBounds.x += (e.x - savedX);
				windowBounds.y += (e.y - savedY);
				Wind.instance.setBounds(windowBounds);
			}
		}
		
		public void controlResized(Event e) {
			/*
			 * 캔버스의 크기가 변할 때에 width, height의 크기를 갱신한다.
			 */
			Point p = getSize();
			width = p.x;
			height = p.y;
			centerImage();
		}
		
		public void paintControl(PaintEvent e) {
			drawImage(e.gc);
		}
		
		public void handleEvent(Event e) {
			switch(e.type){
			case SWT.Resize:
				controlResized(e);
				break;
			case SWT.MouseWheel:
				if(e.count > 0)
					mouseWheelUp(e);
				else
					mouseWheelDown(e);
				break;
			case SWT.MouseDown:
				mouseDown(e);
				break;
			case SWT.MouseUp:
				mouseUp(e);
				break;
			}
		}
		
		public boolean checkImage(){
			if(data.image == null){
				Wind.instance.openFileDialog();
				return false;
			}
			return true;
		}

		public void mouseWheelUp(Event e){
			if(checkImage())
				Wind.instance.openPreviousFile();
		}
		
		public void mouseWheelDown(Event e){
			if(checkImage())
				Wind.instance.openNextFile();
		}
		
		public void dragEnter(DropTargetEvent e) {}
		
		public void dragLeave(DropTargetEvent e) {}
		
		public void dragOperationChanged(DropTargetEvent e) {}
		
		public void dragOver(DropTargetEvent e) {}
		
		public void drop(DropTargetEvent e) {
			for(TransferData data : e.dataTypes){
				if(FileTransfer.getInstance().isSupportedType(data)){
					String[] files = (String[])e.data;
					Wind.instance.openFile(files[0]);
				}
			}
		}
		
		public void dropAccept(DropTargetEvent e) {}
	}
	
	/**
	 * 이미지의 확대 비율, 위치 등의 정보를 초기화한다.
	 * @see #clear()
	 */
	public void reset(){
		data.reset();
		centerImage();
		packWithTopWindow();
		redraw();
	}
	
	/**
	 * 이미지를 화면 중앙에 위치시킨다.
	 */
	private void centerImage(){
		if(data.image != null){
			/*
			 * 그림의 크기가 화면보다 같거나 작을 경우, 그림을 중앙으로 위치시킨다.
			 */
			if(width >= data.imageWidthMag)
				viewportX = (width - data.imageWidthMag) / 2;
			if(height >= data.imageHeightMag)
				viewportY = (height - data.imageHeightMag) / 2;
			/*
			 * 그림이 화면에서 벗어나 있을 경우 0, 0의 위치로 초기화한다.
			 */
			if(viewportX < 0 || data.imageWidthMag > width) viewportX = 0;
			if(viewportY < 0 || data.imageHeightMag > height) viewportY = 0;
		}
	}
	
	private void cropCanvas(){
		/*
		 * (width|height)Hint는 공간을 '가능한 한' 확보하도록 요청하는 한편
		 * minimum(Width|Height)은 반드시 공간을 확보하게 하기 때문에
		 * 스크린 크기를 넘어가는 minumum 값은 Wind.statusBar를 때때로 감출 수도 있다.
		 */
		//layoutData.minimumWidth = data.imageWidthMag;
		//layoutData.minimumHeight = data.imageHeightMag;
		layoutData.widthHint = data.imageWidthMag;
		layoutData.heightHint = data.imageHeightMag;
	}
	
	public void center(){
		centerImage();
		cropCanvas();
		redraw();
	}

	/**
	 * 이미지를 제거하고 캔버스를 초기화한다.
	 * @see #reset()
	 */
	public void clear(){
		data.clear();
		redraw();
	}
	
	public void zoomIn(){
		if (data.zoomIn()){
			packWithTopWindow();
			redraw();
		}
	}
	
	public void zoomOut(){
		if(data.zoomOut()){
			packWithTopWindow();
			redraw();
		}
	}
	
	public boolean getFitToScreen(){
		return fitToScreen;
	}
	
	public void fitToScreen(){
		fitToScreen = !fitToScreen;
		redraw();
	}
	
	private void packWithTopWindow(){
		if(data.magnification >= 1){
			cropCanvas();
			Wind.instance.pack(false);
		}
	}
	
	public void showImage(Image image){
		data.clear();
		data.setImage(image);

		if(!fitToScreen) cropCanvas();
		
		/*
		 * 화면의 크기가 그림보다 클 경우 그림을 중앙에 위치시킨다.
		 */
		centerImage();
		/*
		 * pack()으로도 실행되지만 이전 이미지와 사이즈가 같을 경우
		 * pack()은 실행되지 않는다.
		 */
		redraw();
	}
	
	public Image getImage(){
		return data.image;
	}
}