package ni2.likewind.ui;

import java.io.File;

import ni2.likewind.image.ImageLoader;
import ni2.likewind.image.ImageWriter;
import ni2.likewind.io.DirectoryCrawler;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

/**
 * @author ni2 (netisinfinite@gmail.com)
 * @version 0.1 11/14/06
 * This code licensed under the terms of the GPL.  See doc/LICENSE.
 */
public class Wind{
	public static Wind instance;
	private static final Display DISPLAY = new Display();
	public static final Cursor CURSOR_GRAB = new Cursor(DISPLAY, SWT.CURSOR_SIZEALL);
	//private static final int SHELL_STYLE_CLASSIC = SWT.NONE;
	private final Shell shell = new Shell(DISPLAY);
	private final ImageCanvas canvas = new ImageCanvas(shell, SWT.NONE);
	private StatusBar statusBar;
	private final MenuListener listener = new MenuListener();
	private final DirectoryCrawler crawler = new DirectoryCrawler();
	
	
	public Wind(){
		createAndShowUI();
		instance = this;
	}
	
	private void createAndShowUI(){
		shell.setText("Like Wind");
		GridLayout layout = new GridLayout(1, false);
		layout.horizontalSpacing = 0;
		layout.verticalSpacing = 0;
		layout.marginWidth = 0;
		layout.marginHeight = 0;
		shell.setLayout(layout);
		
		canvas.setLayoutData(new GridData(GridData.FILL_BOTH));
		canvas.setBackground(DISPLAY.getSystemColor(SWT.COLOR_BLACK));
		
		createMainMenu();
		createPopupMenu();
		createStatusBar();
		
		//SHELL.setSize(300,400);
		shell.open();
	}
	
	private void createStatusBar(){
		statusBar = new StatusBar(shell, SWT.BORDER);
		statusBar.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
	}
	
	private void createMainMenu(){
		Menu menu = new Menu(shell, SWT.BAR);
		MenuItem item;
		
		item = new MenuItem(menu, SWT.CASCADE);
		item.setText(WindMenu.FILE);
		Menu menuFile = new Menu(item);
		item.setMenu(menuFile);

		item = new MenuItem(menuFile, SWT.PUSH);
		item.setText(WindMenu.OPEN);
		item.setAccelerator(WindMenu.OPEN_AK);
		item.addListener(SWT.Selection, listener);

		item = new MenuItem(menuFile, SWT.PUSH);
		item.setText(WindMenu.SAVE_AS);
		item.setAccelerator(WindMenu.SAVE_AS_AK);
		item.addListener(SWT.Selection, listener);
		
		item = new MenuItem(menuFile, SWT.PUSH);
		item.setText(WindMenu.NEXT_IMAGE);
		item.setAccelerator(WindMenu.NEXT_IMAGE_AK);
		item.addListener(SWT.Selection, listener);
		
		item = new MenuItem(menuFile, SWT.PUSH);
		item.setText(WindMenu.PREVIOUS_IMAGE);
		item.setAccelerator(WindMenu.PREVIOUS_IMAGE_AK);
		item.addListener(SWT.Selection, listener);
		
		item = new MenuItem(menuFile, SWT.PUSH);
		item.setText(WindMenu.EXIT);
		item.setAccelerator(WindMenu.EXIT_AK);
		item.addListener(SWT.Selection, listener);
		
		item = new MenuItem(menu, SWT.CASCADE);
		item.setText(WindMenu.VIEW);
		Menu menuView = new Menu(item);
		item.setMenu(menuView);
		
		item = new MenuItem(menuView, SWT.PUSH);
		item.setText(WindMenu.ZOOM_IN);
		item.setAccelerator(WindMenu.ZOOM_IN_AK);
		item.addListener(SWT.Selection, listener);
		
		item = new MenuItem(menuView, SWT.PUSH);
		item.setText(WindMenu.ZOOM_OUT);
		item.setAccelerator(WindMenu.ZOOM_OUT_AK);
		item.addListener(SWT.Selection, listener);
		
		item = new MenuItem(menuView, SWT.PUSH);
		item.setText(WindMenu.FIT_TO_SCREEN);
		item.setAccelerator(WindMenu.FIT_TO_SCREEN_AK);
		item.addListener(SWT.Selection, listener);

		item = new MenuItem(menuView, SWT.PUSH);
		item.setText(WindMenu.ACTUAL_SIZE);
		item.setAccelerator(WindMenu.ACTUAL_SIZE_AK);
		item.addListener(SWT.Selection, listener);
		
		item = new MenuItem(menuView, SWT.PUSH);
		item.setText(WindMenu.CLEAR);
		item.setAccelerator(WindMenu.CLEAR_AK);
		item.addListener(SWT.Selection, listener);
		
		item = new MenuItem(menuView, SWT.PUSH);
		item.setText(WindMenu.RESET);
		item.setAccelerator(WindMenu.RESET_AK);
		item.addListener(SWT.Selection, listener);
		
		item = new MenuItem(menuView, SWT.PUSH);
		item.setText(WindMenu.FULL_SCREEN);
		item.setAccelerator(WindMenu.FULL_SCREEN_AK);
		item.addListener(SWT.Selection, listener);
		
		shell.setMenuBar(menu);
	}
	
	private void createPopupMenu(){
		final Menu menu = new Menu(canvas);
		MenuItem item;
		
		item = new MenuItem(menu, SWT.PUSH);
		item.setText(WindMenu.OPEN);
		item.addListener(SWT.Selection, listener);
		
		item = new MenuItem(menu, SWT.PUSH);
		item.setText(WindMenu.SAVE_AS);
		item.addListener(SWT.Selection, listener);
		
		item = new MenuItem(menu, SWT.PUSH);
		item.setText(WindMenu.EXIT);
		item.addListener(SWT.Selection, listener);
		
		new MenuItem(menu, SWT.SEPARATOR);
		
		item = new MenuItem(menu, SWT.PUSH);
		item.setText(WindMenu.ZOOM_IN);
		item.addListener(SWT.Selection, listener);

		item = new MenuItem(menu, SWT.PUSH);
		item.setText(WindMenu.ZOOM_OUT);
		item.addListener(SWT.Selection, listener);

		item = new MenuItem(menu, SWT.PUSH);
		item.setText(WindMenu.ACTUAL_SIZE);
		item.addListener(SWT.Selection, listener);
		
		item = new MenuItem(menu, SWT.PUSH);
		item.setText(WindMenu.RESET);
		item.addListener(SWT.Selection, listener);
		
		item = new MenuItem(menu, SWT.PUSH);
		item.setText(WindMenu.CLEAR);
		item.addListener(SWT.Selection, listener);
		
		canvas.addListener(SWT.MenuDetect, new Listener() {
			public void handleEvent(Event event) {
				menu.setLocation(event.x, event.y);
				menu.setVisible(true);
			}
		});
	}
	
	public void saveFile(String fileName){
		ImageWriter writer = new ImageWriter(fileName, canvas.getImage());
		if(!writer.write()) alert("지원하지 않는 파일 형식입니다.");
	}
	
	public void openFile(File file){
		if(file != null) openFile(file.getPath());
	}
	
	public void openFile(String fileName){
		if(fileName != null)
			try{
				Image image = ImageLoader.loadImage(fileName);
				
				if(image == null){
					alert("지원하지 않는 파일 형식입니다.");
					return;
				}
				
				Rectangle size = image.getBounds();
				canvas.showImage(image);
				crawler.crawl(fileName);
				shell.setText(fileName);
				statusBar.putIndex(crawler.count(), crawler.index());
				statusBar.putMessage(fileName);
				statusBar.putSize(size.width, size.height);
				statusBar.apply();
				
				if(! canvas.getFitToScreen()) pack(false);
			}catch(Exception e){
				e.printStackTrace();
			}
	}
	
	public void saveFileDialog(){
		FileDialog fd = new FileDialog(shell);
		String fileName = fd.open();
		
		if(fileName != null) saveFile(fileName);
	}
	
	public void openFileDialog(){
		FileDialog fd = new FileDialog(shell);
		fd.setFilterExtensions(ImageLoader.SUPPORTED_FILE_TYPES_EXT);
		//fd.setFilterNames(ImageCanvas.SUPPORTED_FILE_TYPES_DESCRIPTION);
		String fileName = fd.open();
		
		if(fileName != null) openFile(fileName);
	}
	
	public void openNextFile(){
		openFile(crawler.next());
	}
	
	public void openPreviousFile(){
		openFile(crawler.previous());
	}
	
	public void alert(String message){
		MessageBox mb = new MessageBox(shell, SWT.NO);
		mb.setText("알림");
		mb.setMessage(message);
		mb.open();
	}
	
	public void maxmize(){
		shell.setMaximized(shell.getMaximized()? false : true);
	}
	
	public boolean isMaxmized(){
		return shell.getMaximized();
	}
	
	public Rectangle getBounds(){
		return shell.getBounds();
	}
	
	public void setBounds(Rectangle bounds){
		shell.setBounds(bounds);
	}
	
	/**
	 * 윈도우의 크기를 그림(ImageCanvas)에 맞추어 재조정한다. 
	 * @param force 최대화 및 풀스크린 상태에서도 강제로 실행할 것인지를 결정한다.
	 * @return pack이 성공적으로 실행되었는지의 여부
	 */
	public void pack(boolean force){
		if(force || !shell.getMaximized()) shell.pack();
	}
	
	static class MenuListener implements Listener{
		public void handleEvent(Event e) {
			if(e.widget instanceof MenuItem){
				String menu = ((MenuItem)e.widget).getText();
				if(menu == WindMenu.OPEN){
					instance.openFileDialog();
				}else if(menu == WindMenu.SAVE_AS){
					instance.saveFileDialog();
				}else if(menu == WindMenu.EXIT){
					instance.shell.dispose();
				}else if(menu == WindMenu.ZOOM_IN){
					instance.canvas.zoomIn();
				}else if(menu == WindMenu.ZOOM_OUT){
					instance.canvas.zoomOut();
				}else if(menu == WindMenu.FIT_TO_SCREEN){
					instance.canvas.fitToScreen();
				}else if(menu == WindMenu.ACTUAL_SIZE){
					instance.canvas.reset();
				}else if(menu == WindMenu.NEXT_IMAGE){
					instance.openNextFile();
				}else if(menu == WindMenu.PREVIOUS_IMAGE){
					instance.openPreviousFile();
				}else if(menu == WindMenu.FULL_SCREEN){
					instance.maxmize();
				}else if(menu == WindMenu.CLEAR){
					instance.canvas.clear();
					instance.statusBar.reset();
				}else if(menu == WindMenu.RESET){
					instance.canvas.reset();
				}
			}
			
		}
	}
	
	interface WindMenu{
		public static final String FILE = "&File";
		public static final String VIEW = "&View";
		public static final String OPEN = "&Open\tO";
		public static final int OPEN_AK = 'O';
		/*
		 * 이미지를 덮어 씌우는 실수를 피하기 위해, LikeWind에서는 Save As 기능만을 지원한다.
		 * TODO 기존 이미지를 덮어 씌우는 것을 (File Dialog 에서까지) 원천적으로 방지 
		 */
		public static final String SAVE_AS = "&Save As\tS";
		public static final int SAVE_AS_AK = 'S';
		public static final String EXIT = "E&xit\tX";
		public static final int EXIT_AK = 'X';
		public static final String ZOOM_IN = "&Zoom In\t+";
		public static final int ZOOM_IN_AK = SWT.KEYPAD_ADD | '+';
		public static final String ZOOM_OUT = "&Zoom Out\t-"; 
		public static final int ZOOM_OUT_AK = SWT.KEYPAD_SUBTRACT | '-';
		public static final String PREVIOUS_IMAGE = "&Previous Image\tPgUp";
		public static final String FIT_TO_SCREEN = "&Fit to Screen\tF";
		public static final int FIT_TO_SCREEN_AK = 'F';
		public static final int PREVIOUS_IMAGE_AK = SWT.PAGE_UP;
		public static final String ACTUAL_SIZE = "&Actual Size\t=";
		public static final int ACTUAL_SIZE_AK = SWT.KEYPAD_EQUAL;
		public static final String NEXT_IMAGE = "Next Image\tPgDn";
		public static final int NEXT_IMAGE_AK = SWT.PAGE_DOWN;
		public static final String CLEAR = "&Clear\tC";
		public static final int CLEAR_AK = 'C';
		public static final String RESET = "&Reset\tR";
		public static final int RESET_AK = 'R';
		public static final String FULL_SCREEN = "&Full Screen\tEnter";
		public static final int FULL_SCREEN_AK = SWT.CR;
		public static final int MOVE_MODE_AK = SWT.ALT;
	}
	
	public void mainLoop(){
		while(!shell.isDisposed()){
			if(!DISPLAY.readAndDispatch())
				DISPLAY.sleep();
		}
		DISPLAY.dispose();
	}
}
 