package net.kldp.zeta.monimoni;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;

import net.kldp.zeta.monimoni.data.Wiki;
import net.kldp.zeta.monimoni.data.Wiki.Action;

import org.apache.commons.httpclient.DefaultHttpMethodRetryHandler;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.httpclient.URI;
import org.apache.commons.httpclient.URIException;
import org.apache.commons.httpclient.cookie.CookiePolicy;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.params.HttpMethodParams;

public class Client {
	private final HttpClient client = new HttpClient();
	private HttpMethod method = new GetMethod();
	private final Wiki wiki;
	
	public Client(Wiki wiki){
		this.wiki = wiki;
		setURI(wiki.getURI());
		
		method.getParams().setParameter(HttpMethodParams.RETRY_HANDLER, 
	    		new DefaultHttpMethodRetryHandler(3, false));
	}
	
	public void setURI(URI URI){
		try {
			method.setURI(URI);
		} catch (URIException e) {
			e.printStackTrace();
		}
	}
	
	public String getRaw(String page){
		setURI(wiki.getURI(page));
		method.setQueryString(new NameValuePair[]{Action.RAW.toNameValuePair()});
		String result = encode(executeMethod());
		method.setQueryString(new NameValuePair[]{});
		return result;
	}
	
	//FIXME : 오직 LoginTest를 위해 존재
	public String getHistory(String page){
		setURI(wiki.getURI(page));
		method.setQueryString(new NameValuePair[]{Action.HISTORY.toNameValuePair()});
		String result = encode(executeMethod());
		method.setQueryString(new NameValuePair[]{});
		return result;
	}
	
	private String read(InputStream input){
		StringBuilder sb = new StringBuilder();
		try {
			BufferedReader bufferedReader = new BufferedReader(
					new InputStreamReader(input,"8859_1"));

			while(true){
				String str = bufferedReader.readLine();
				if(str == null) break;
				sb.append(str);
				sb.append('\n');
			}
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return sb.toString();
	}
	
	public boolean login(String id, String password){
		boolean result = false;
		PostMethod method = new PostMethod();
		try {
			method.setURI(wiki.getURI("FrontPage"));
			method.setRequestBody(new NameValuePair[]{
					new NameValuePair("action","userform"),
					new NameValuePair("login_id",id),
					new NameValuePair("password",password)
			});
			method.getParams().setCookiePolicy(CookiePolicy.BROWSER_COMPATIBILITY);
			client.executeMethod(method);
			result = true;
			method.releaseConnection();
		} catch (URIException e) {
			e.printStackTrace();
		} catch (HttpException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return result;
	}
	
	public boolean save(String page, String source, String summary){
		boolean result = false;
		PostMethod method = new PostMethod();
		try {
			method.setURI(wiki.getURI(page + "#preview"));
			method.setRequestBody(new NameValuePair[]{
					new NameValuePair("content",source),
					new NameValuePair("comment",summary),
					new NameValuePair("action","savepage")
					//new NameValuePair("datestamp",System.currentTimeMillis() + "")
					/*
					 * TODO 문서의 최종 편집일을 datestamp로 받아 conflict check를 하는 로직이 필요.
					 */
			});
			client.executeMethod(method);
			result = true;
			method.releaseConnection();
		} catch (URIException e) {
			e.printStackTrace();
		} catch (HttpException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return result;
	}
	
	public String encode(String source){
		String result = null;
		try {
			result = new String(source.getBytes("8859_1"), wiki.getEncoding());
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		return result;
	}
	
	public String executeMethod(){
		String result = null;
		
		try {
			client.executeMethod(method);
			result = read(method.getResponseBodyAsStream());
		} catch (HttpException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		method.releaseConnection();
		
		return result;
	}
}
