package net.kldp.zeta.monimoni.data;

import net.kldp.zeta.monimoni.Client;

import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.httpclient.URI;
import org.apache.commons.httpclient.URIException;

public class Wiki {
	private static final String URI_PATH_SEPARATOR = "/";
	private static final String URI_HTTP_SCHEME_IDENTIFIER = "http://";
	private static final String URI_SEARCHPART_STARTER = "?";
	
	private Client client;
	private String name;
	private URI URI;
	private Type type;
	private String encoding;
	
	public enum Type{
		MONIWIKI ("Moniwiki", ".moniwiki");
		
		String name;
		String extension;
		/**
		 * 
		 * @param name 통용되는 타입의 이름
		 * @param extension 오프라인 파일의 확장자, Colorer 라이브러리의 파싱과도 관련이 있음 
		 * @see net.kldp.zeta.moniwiki.ui.tab.PageTabFolder.configViewer(org.eclipse.jface.text.source.SourceViewer);
		 */
		Type(String name, String extension){
			this.name = name;
			this.extension = extension;
		}
		public String getName(){
			return name;
		}
		public String getExtension(){
			return extension;
		}
		public static Type byName(String name){
			Type result = null;
			for(Type t : Type.values()){
				if(t.name.equals(name)){
					result = t;
					break;
				}
			}
			return result;
		}
	}
	
	/* http://moniwiki.sourceforge.net/wiki.php/Ko~MoniWikiActions 참고
	 */
	public enum Action{
		VIEW ("show"),
		PREVIEW ("preview"),
		HISTORY ("info"),
		RAW ("raw");
		
		String modifier;
		Action(String modifier){
			this.modifier = modifier;
		}
		public String getActionModifier(){
			return "action=" + modifier;
		}
		public NameValuePair toNameValuePair(){
			return new NameValuePair("action",modifier);
		}
	}
	
	public Wiki(String URI, String encoding, Type type) throws URIException{
		this.encoding = encoding;
		this.type = type;
		this.name = prevalidateURI(URI);
		this.URI = createURI(this.name);
	}
	
	private URI createURI(String URI) throws URIException{
		URI result = new URI(URI, false);
		return result;
	}
	
	private String prevalidateURI(String URI){
		if(! URI.startsWith(URI_HTTP_SCHEME_IDENTIFIER))
			URI = URI_HTTP_SCHEME_IDENTIFIER + URI;
		if(URI.endsWith(URI_PATH_SEPARATOR))
			URI = URI.substring(0, URI.length() - 1);
		//TODO wiki.php로 끝나지 않는 moniwiki가 있을 수 있음,
		//Wiki Type별  WikiURIValidator 클래스를 이용한 처리
		if(type == Type.MONIWIKI && (! URI.endsWith("wiki.php")))
			URI += "/wiki.php";
		return URI;
	}
	
	public Client getClient(){
		if(client == null)
			client = new Client(this);
		return client;
	}
	
	public String getName(){
		return name;
	}
	
	public String getEncoding(){
		return encoding;
	}
	
	public URI getURI(){
		return URI;
	}
	
	public URI getURI(String keyword){
		URI result = null;
		try {
			result = createURI(URI + URI_PATH_SEPARATOR + keyword);
		} catch (URIException e) {
			e.printStackTrace();
		}
		return result;
	}
	
	public URI getURI(String keyword, Action action){
		URI result = null;
		StringBuilder sb = new StringBuilder();
		sb.append(URI);
		sb.append(URI_PATH_SEPARATOR);
		sb.append(keyword);
		sb.append(URI_SEARCHPART_STARTER);
		sb.append(action.getActionModifier());
		
		try {
			result = createURI(sb.toString());
		} catch (URIException e) {
			e.printStackTrace();
		}
		return result;
	}
}
