package net.kldp.zeta.monimoni.manager;

import java.util.ArrayList;
import java.util.List;

import net.kldp.zeta.monimoni.data.Wiki;
import net.kldp.zeta.monimoni.data.WikiPage;
import net.kldp.zeta.monimoni.event.ChangeEvent;
import net.kldp.zeta.monimoni.event.ChangeListener;

public class WikiManager {
	private static WikiManager instance;
	private List<Wiki> wikis = new ArrayList<Wiki>();
	private List<ChangeListener> listeners = new ArrayList<ChangeListener>();
	
	public static final int ADD = 1;
	public static final int REMOVE = 2;
	public static final int OPEN_PAGE = 3;
	public static final int CLOSE_PAGE = 4;
	
	private WikiManager(){
		
	}
	
	public static WikiManager instance(){
		if(instance == null){
			instance = new WikiManager();
		}
		return instance;
	}
	
	public void addChangeListener(ChangeListener listener){
		listeners.add(listener);
	}
	
	public List<Wiki> getItems(){
		return wikis;
	}
	
	public void add(Wiki w){
		wikis.add(w);
		notifyToListener(ADD, w);
	}
	
	public void add(WikiPage p){
		notifyToListener(OPEN_PAGE, p);
	}
	
	private void notifyToListener(int status, Object o){
		for(ChangeListener listener : listeners){
			listener.stateChanged(
					new ChangeEvent(status, o)
					);
		}
	}
	
	public Wiki get(int index){
		return wikis.get(index);
	}
	
	public void remove(Wiki wiki){
		notifyToListener(REMOVE, get(wikis.indexOf(wiki)));
		wikis.remove(wiki);
	}
	
	public void remove(WikiPage p){
		notifyToListener(CLOSE_PAGE, p);
	}
	
	public void remove(int index){
		notifyToListener(REMOVE, get(index));
		wikis.remove(index);
	}
}
