package net.kldp.zeta.monimoni.ui;

import net.kldp.zeta.commons.resource.Messages;
import net.kldp.zeta.monimoni.action.AddWikiAction;
import net.kldp.zeta.monimoni.action.CloseWikiPageAction;
import net.kldp.zeta.monimoni.action.ExitAction;
import net.kldp.zeta.monimoni.action.LoginWikiAction;
import net.kldp.zeta.monimoni.action.OpenWikiPageAction;
import net.kldp.zeta.monimoni.action.RemoveWikiAction;
import net.kldp.zeta.monimoni.manager.WikiManager;
import net.kldp.zeta.monimoni.ui.tab.PageTabFolder;
import net.kldp.zeta.monimoni.ui.tree.WikiTree;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.window.ApplicationWindow;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

public class ClientWindow extends ApplicationWindow{

	public ClientWindow(Shell arg0) {
		super(null);
		addMenuBar();
	}
	
	private PageTabFolder pageFolder;
	private MenuManager fileMenu;
	
	public PageTabFolder getPageFolder(){
		return pageFolder;
	}
	
	private final Separator separatorAction = new Separator(); 
	private Action openWikiPageAction;
	private Action removeWikiAction;
	private Action closeWikiPageAction;
	private Action addWikiAction;
	private Action loginWikiAction;
	
	@Override
	protected MenuManager createMenuManager(){
		MenuManager menuBar = new MenuManager();
		fileMenu = new MenuManager(Messages.get("FileMenuText"));
		MenuManager aboutMenu = new MenuManager(Messages.get("AboutMenuText"));
		menuBar.add(fileMenu);
		menuBar.add(aboutMenu);
		//createMenuManager()에서 추가되는 Action이 하나도 없는 MenuManager는 후에 추가되더라도 window에서 감추어지게 된다.
		fileMenu.add(addWikiAction = new AddWikiAction());
		return menuBar;
	}
	
	private MenuManager createTreePopupMenuManager(){
		MenuManager popup = new MenuManager();
		popup.add(addWikiAction);
		popup.add(removeWikiAction);
		popup.add(separatorAction);
		popup.add(loginWikiAction);
		popup.add(separatorAction);
		popup.add(openWikiPageAction);
		popup.add(closeWikiPageAction);
		return popup;
	}
	
	@Override
	protected Control createContents(Composite parent){
		getShell().setText(Messages.get("Title"));
		Composite contents = new Composite(parent, SWT.FLAT);
		GridLayout layout = new GridLayout(2, false);
		layout.marginWidth = 0;
		layout.marginHeight = 0;
		layout.verticalSpacing = 0;
		layout.horizontalSpacing = 5;
		contents.setLayout(layout);
		
		WikiTree wikiTree = new WikiTree(contents, SWT.NONE);
		fileMenu.add(removeWikiAction = new RemoveWikiAction(wikiTree.getTreeWidget()));
		fileMenu.add(separatorAction);
		fileMenu.add(loginWikiAction = new LoginWikiAction(wikiTree.getTreeWidget()));
		fileMenu.add(separatorAction);
		fileMenu.add(openWikiPageAction = new OpenWikiPageAction(wikiTree.getTreeWidget(), this));
		fileMenu.add(closeWikiPageAction = new CloseWikiPageAction(wikiTree.getTreeWidget(), this));
		fileMenu.add(separatorAction);
		fileMenu.add(new ExitAction(this));
		wikiTree.setPopupMenu(createTreePopupMenuManager());
		GridData data = new GridData(GridData.FILL_VERTICAL);
		data.widthHint = 200;
		wikiTree.getTreeWidget().setLayoutData(data);
		WikiManager manager = WikiManager.instance();
		manager.addChangeListener(wikiTree);
		
		//Sash sash = new Sash(contents, SWT.VERTICAL);
		//data = new GridData(GridData.FILL_BOTH);
		//sash.setLayoutData(data);
		
		pageFolder = new PageTabFolder(contents, SWT.FLAT | SWT.CLOSE);
		data = new GridData(GridData.FILL_BOTH);
		data.widthHint = 400;
		pageFolder.getTabFolderWidget().setLayoutData(data);
		
		return contents;
	}
}