package net.kldp.zeta.monimoni.ui.tab;

import net.kldp.zeta.monimoni.data.Wiki;
import net.kldp.zeta.monimoni.data.WikiPage;
import net.kldp.zeta.monimoni.data.Wiki.Action;

import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.ITextListener;
import org.eclipse.jface.text.TextEvent;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.program.Program;

public class PageTabItem{
	private Document document;
	private Wiki wiki;
	private WikiPage page;
	private SourceViewer editor;
	private CTabItem pageItem;
	private CTabItem editTab;
	private TabItemBrowserPair[] tabItems;
	
	class TabListener implements SelectionListener{		
		public void widgetDefaultSelected(SelectionEvent e) {}
		public void widgetSelected(SelectionEvent e) {
			for(TabItemBrowserPair pair : tabItems){
				if(e.item == pair.getTabItem())
					pair.setUrl(wiki.getURI(page.getName(), pair.getAction()).toString());
			}
		}
	}
	
	class EditorListener implements ITextListener{
		public void textChanged(TextEvent arg0) {
			editTab.setText(editTab.getText() + "*");
		}

	}
	
	public CTabItem getTabItem(){
		return pageItem;
	}
	
	public PageTabItem(WikiPage page, PageTabFolder parent){
		this.wiki = page.getParent();
		this.page = page;
		pageItem = new CTabItem(parent.getTabFolderWidget(), SWT.NONE);
		CTabFolder folder = new CTabFolder(parent.getTabFolderWidget(), SWT.BOTTOM);
		editor = new SourceViewer(folder, null, SWT.MULTI | SWT.WRAP | SWT.V_SCROLL);
		document = new Document();
		editor.setDocument(document);
		editTab = new CTabItem(folder, SWT.NONE);
		editTab.setText("edit");
		editTab.setControl(editor.getControl());
		editor.addTextListener(new EditorListener());
		this.tabItems = new TabItemBrowserPair[]{
				//new TabItemBrowserPair(folder, "preview", Action.PREVIEW),
				//new TabItemBrowserPair(folder, "original", Action.VIEW),
				//new TabItemBrowserPair(folder, "history", Action.HISTORY)
		};

		folder.addSelectionListener(new TabListener());
		
		pageItem.setText(page.getName());
		pageItem.setControl(folder);
		folder.setSelection(0);
	}
	
	public void dispose(){
		for(TabItemBrowserPair pair : tabItems){
			pair.dispose();
		}
		editor.getTextWidget().dispose();
		editTab.dispose();
		pageItem.dispose();
	}
	
	public WikiPage getWikiPage(){
		return page;
	}
	
	public Document getDocument(){
		return document;
	}
	
	public SourceViewer getEditor(){
		return editor;
	}
}

class TabItemBrowserPair{
	private CTabFolder parent;
	private CTabItem item;
	private Browser browser;
	private Action action;
	
	public TabItemBrowserPair(CTabFolder parent, String name, Action action){
		this.parent = parent;
		item = new CTabItem(parent, SWT.NONE);
		item.setText(name);
		item.setControl(browser);
		this.action = action;
	}
	
	/*
	 * Browser widget의 초기 생성 속도 및 리소스 문제 때문에
	 * 첫 실행시에 위젯을 생성하도록 제어함
	 * FIXME 정상 동작하지 않음
	 */
	private Browser getBrowser(){
		if(browser == null)
			browser = new Browser(parent, SWT.NONE);
		return browser;
	}
	
	public void setUrl(String URI){
		//getBrowser().setUrl(URI);
		openExternalBrowser(URI);
	}
	
	public void openExternalBrowser(String URI){
		Program.launch(URI);
	}
	
	public CTabItem getTabItem(){
		return item;
	}
	
	public Action getAction(){
		return action;
	}
	
	public void dispose(){
		if(browser != null)
			browser.dispose();
		item.dispose();
	}
}