package net.kldp.zeta.wherithlov.data;

import java.util.ArrayList;
import java.util.List;

import net.kldp.zeta.wherithlov.manager.IrcConnection;


/**
 * @author ni2 (netisinfinite@gmail.com)
 * @version 0.1 11/25/06
 * This code licensed under the terms of the GPL.  See doc/LICENSE.
 */
public class Channel{
	public final IrcConnection connection;
	public final Server server;
	public final String name;
	private List<IChannelListener> listeners = new ArrayList<IChannelListener>();
	private List<AdvUser> users = new ArrayList<AdvUser>();
	
	public Channel(IrcConnection connection, Server server, String channel){
		this.connection = connection;
		this.server = server;
		this.name = channel;
	}
	
	public boolean equals(Channel channel){
		return (channel.server.equals(server) && channel.name.equals(name)) ? true : false;
	}
	
	public void addListener(IChannelListener listener){
		listeners.add(listener);
	}
	
	public void addUser(AdvUser user){
		//FIXME : users 리스트에 변화를 줄 것인지를 결정해야 한다.
		userAdded(user);
	}
	
	public List<AdvUser> getUsers(){
		return users;
	}
	
	public void setUsers(List<AdvUser> users){
		this.users = users;
		userListChanged();
	}
	
	public void removeUser(AdvUser user, String message){
		//FIXME : users 리스트에 변화를 줄 것인지를 결정해야 한다.
		userRemoved(user, message);
	}
	
	public void setTopic(String topic){
		topicChanged(topic);
	}
	
	private void topicChanged(String topic) {
		for(IChannelListener l : listeners){
			l.topicChanged(topic);
		}
	}

	public void recieveMessage(String message){
		messageRecieved(message);
	}
	
	public void sendMessage(String message){
		connection.sendMessage(name, message);
		messageSended(message);
		//메시지 전송시 내 메시지는 onMessage 이벤트를 발생시키지 않으므로, 임의로 발생시킨다.
		messageRecieved("<" + connection.getName() + "> " + message);
	}
	
	private void userListChanged(){
		for(IChannelListener l : listeners){
			l.userListChanged();
		}
	}

	private void userAdded(AdvUser user){
		for(IChannelListener l : listeners){
			l.userAdded(user);
		}
	}
	
	private void userRemoved(AdvUser user, String message){
		for(IChannelListener l : listeners){
			l.userRemoved(user, message);
		}
	}
	
	private void messageSended(String message){
		for(IChannelListener l : listeners){
			l.messageSended(message);
		}
	}
	
	private void messageRecieved(String message){
		for(IChannelListener l : listeners){
			l.messageRecieved(message);
		}
	}
	
	public void part(){
		//FIXME GUI 갱신 등 추가적인 처리가 필요하다.
		connection.partChannel(name);
	}
}
