package net.kldp.zeta.wherithlov.manager;

import java.util.ArrayList;
import java.util.List;

import net.kldp.zeta.commons.resource.Messages;
import net.kldp.zeta.wherithlov.data.AdvUser;
import net.kldp.zeta.wherithlov.data.Channel;
import net.kldp.zeta.wherithlov.data.ChannelCollection;
import net.kldp.zeta.wherithlov.data.IChannelCollectionListener;
import net.kldp.zeta.wherithlov.data.Player;
import net.kldp.zeta.wherithlov.data.Sender;
import net.kldp.zeta.wherithlov.data.Server;
import net.kldp.zeta.wherithlov.ui.MainPane;

/**
 * 전체 연결을 관리. PircBotWrapper와 1대 다로 매칭되는 컨트롤러.
 * FIXME 현재는 다중 접속 관리를 위한 설정 되어 있지 않음
 * 
 * @author ni2 (netisinfinite@gmail.com)
 * @version 0.1 11/25/06
 * This code licensed under the terms of the GPL.  See doc/LICENSE.
 */
public class IrcClient {
	private final MainPane pane;
	private final List<IrcConnection> connections = new ArrayList<IrcConnection>();
	private final ChannelCollection channels = new ChannelCollection();
	
	public IrcClient(MainPane pane){
		this.pane = pane;
		channels.addListener((IChannelCollectionListener) pane);
	}
	
	public void connect(Player player, Server server, String[] initChannels){
		IrcConnection conn = new PircBotWrapper(this);//FIXME 팩토리 클래스를 만들어 이용할 것
		if(conn.connect(player, server, initChannels)){
			connections.add(conn);
			conn.addConnectionMessageListener(pane);
			for(String channelName : initChannels){
				Channel c = new Channel(conn, server, channelName);
				conn.joinChannel(c);
				/*
				성공적으로 방에 들어갔는지 여부를 즉시 확인할 수 없으므로 아래 코드는 쓸 수 없다.
				대신 onJoin()을 사용한다.
				if(conn.joinChannel(c)){
					channels.add(c);
				}else{
					//채널이 없는 등의 이유로 들어갈 수 없으면 Channel 객체는 버려진다.
				}
				*/
			}
		}else{
			//연결에 실패하면 IrcConnection 객체는 버려진다.
		}
	}
	
	public void onJoin(Channel channel){
		channels.add(channel);
	}
	
	public void onJoin(Channel channel, Sender sender){
		channel.addUser(new AdvUser("", sender.nick));
	}
	
	/**
	 * 현재 사용자가 방을 빠져나갈 때 실행된다. 
	 * @param channel 사용자가 있던 채널
	 */
	public void onPart(Channel channel){
		channels.remove(channel);
	}
	
	/**
	 * 다른 사용자가 방을 빠져나갈 때 사용된다.
	 * @param user
	 * @param message
	 */
	public void onPart(String user, String message){
		for(Channel channel: channels){
			onPart(user, channel, message);
		}
	}
	
	/**
	 * 다른 사용자가 방을 빠져나갈 때 사용된다.
	 * @param user
	 * @param message
	 */
	public void onPart(String user, Channel channel, String message){
		channel.removeUser(new AdvUser(null, user), message);
	}
	
	public void onUserList(Channel channel, List<AdvUser> users){
		channel.setUsers(users);
	}
	
	public void onTopic(Channel channel, String topic){
		channel.setTopic(topic);
	}
	
	public void disconnectAll(){
		for(IrcConnection irc : connections){
			if(irc.isConnected())
				irc.quitServer(Messages.get("QuitMessage"));
		}
	}
	
	public void onMessage(Channel channel, String senderName, String message){
		onMessage(channel, "<" + senderName + "> " + message);
	}
	
	public void onMessage(Channel channel, String message){
		channel.recieveMessage(message);
	}
	
	public Channel findChannel(Server server, String channelName){
		return channels.get(server, channelName);
	}
}
