package net.kldp.zeta.wherithlov.manager;

import java.net.ConnectException;
import java.util.ArrayList;
import java.util.List;

import net.kldp.zeta.wherithlov.data.AdvUser;
import net.kldp.zeta.wherithlov.data.Channel;
import net.kldp.zeta.wherithlov.data.IConnectionListener;
import net.kldp.zeta.wherithlov.data.Player;
import net.kldp.zeta.wherithlov.data.Sender;
import net.kldp.zeta.wherithlov.data.Server;

import org.jibble.pircbot.NickAlreadyInUseException;
import org.jibble.pircbot.PircBot;
import org.jibble.pircbot.User;

/**
 * PircBot에서 이벤트가 발생하면 Lov 프레임웍 스타일로 치환해준다.
 * @author ni2 (netisinfinite@gmail.com)
 * @version 0.1 11/25/06
 * This code licensed under the terms of the GPL.  See doc/LICENSE.
 */
public class PircBotWrapper extends PircBot implements IrcConnection{
	//A value of null is returned if the PircBot has never tried to connect to a server
	private Server server;
	private Player player;
	public final IrcClient listener;
	
	public PircBotWrapper(IrcClient listener){
        this.listener = listener;
	}
	
	@Override
	protected void onNotice(String sourceNick,
                        String sourceLogin,
                        String sourceHostname,
                        String target,
                        String notice){
		//Sender sender = new Sender(sourceNick, sourceLogin, sourceHostname);
		//listener.onNotice(notice);
		recieveConnectionMessage(notice);
	}

	@Override
	protected void onMessage(String channelName,
                         String senderName,
                         String login,
                         String hostname,
                         String message){
		Channel channel = listener.findChannel(server, channelName);
		listener.onMessage(channel, senderName, message);
		//Sender sender = new Sender(senderName, login, hostname);
		//listener.printMessage(chan, "<" + sender.nick + "> " + message);
	}

	@Override
	protected void onUserList(String channelName,
                          User[] users){
		List<AdvUser> advUsers = new ArrayList<AdvUser>();
		for(User u : users){
			advUsers.add(new AdvUser(u.getPrefix(), u.getNick()));
		}
		Channel channel = listener.findChannel(server, channelName);
		listener.onUserList(channel, advUsers);
	}

	@Override
	protected void onTopic(String channelName,
                       String topic,
                       String setBy,
                       long date,
                       boolean changed){
		Channel channel = listener.findChannel(server, channelName);
		listener.onTopic(channel, topic);
	}

	@Override
	protected void onJoin(String channelName,
		      String nick,
		      String login,
		      String hostname){
		Channel channel = listener.findChannel(server, channelName);
		//새로 만들어진 채널의 경우 목록에 없으므로(IrcClient.connect() 내용 참고)
		if(channel == null) channel = new Channel(this, server, channelName);
		Sender sender = new Sender(nick, login, hostname);
		
		if(player.equals(sender))
			listener.onJoin(channel);
		else
			listener.onJoin(channel, sender);
		
		listener.onMessage(channel, sender.nick, sender.getInfoString() + " 입장");
	}

	@Override
	protected void onPart(String channelName,
		      String user,
		      String login,
		      String hostname){
		Channel channel = listener.findChannel(server, channelName);
		Sender sender = new Sender(user, login, hostname);
		if(player.equals(sender))
			listener.onPart(channel);
		else
			listener.onPart(sender.nick, channel, sender.getInfoString() + " 퇴장");
	}

	@Override
	protected void onQuit(String sourceNick,
                      String sourceLogin,
                      String sourceHostname,
                      String reason){
		Sender sender = new Sender(sourceNick, sourceLogin, sourceHostname);
		if(sender.nick.equals(player.getNick())){
			//TODO 프로그램을 종료하지 않고 Quit하는 방법이 생기면 이 부분을 만들어 넣어야 한다. 
		}else
			listener.onPart(sender.nick, sender.getInfoString() + " 종료 (" + reason + ")");
	}

	@Override
	protected void onServerResponse(int code,
                                String response){
		this.recieveConnectionMessage(code + " " + response);
	}

	@Override
	protected void onServerPing(String response){
		super.onServerPing(response);
		this.recieveConnectionMessage("PING " + response);
		this.recieveConnectionMessage("::: PONG " + response);
	}

	@Override
	public boolean connect(Player player, Server server, String[] channels) {
        this.server = server;
        this.player = player;
		setVerbose(true);
		setLogin(player.getLogin());
		setName(player.getNick());
		try{
			setEncoding(server.encoding);
		}catch(Exception e){
			e.printStackTrace();
		}

		while(! isConnected()){
			try{
				connect(server.host,server.port);
			}catch(NickAlreadyInUseException e){
				e.printStackTrace();
				//FIXME 닉네임 변경을 요구하는 다이얼로그를 띄우도록
				setName(getName() + "_");
			}catch(ConnectException e){
				e.printStackTrace();
				break;
			}catch(Exception e){
				e.printStackTrace();
				break;
			}
		}
		
		return isConnected();
	}
	
	@Override
	public boolean joinChannel(Channel channel){
		joinChannel(channel.name);
		return true;//FIXME 실제 조인이 되었는지, 해당 채널이 없어서 실패하지는 않았는지 확인 요망
	}
	
	public IrcClient getListener(){
		return listener;
	}
	
	private List<IConnectionListener> listeners = new ArrayList<IConnectionListener>();

	@Override
	public void recieveConnectionMessage(String message) {
		connectionMessageRecieved(message);
	}
	
	public void addConnectionMessageListener(IConnectionListener listener){
		listeners.add(listener);
	}
	
	private void connectionMessageRecieved(String message){
		for(IConnectionListener listener : listeners){
			listener.connectionMessageRecieved(message);
		}
	}
}