package net.kldp.zeta.wherithlov.ui;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.List;

import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingUtilities;
import javax.swing.text.Document;

import net.kldp.zeta.commons.resource.Messages;
import net.kldp.zeta.wherithlov.data.AdvUser;
import net.kldp.zeta.wherithlov.data.Channel;
import net.kldp.zeta.wherithlov.data.CommandParser;
import net.kldp.zeta.wherithlov.data.IChannelListener;

/**
 * IRC 채널을 보여주며 채팅 등을 할 수 있는 패널.
 * @author ni2 (netisinfinite@gmail.com)
 * @version 0.2 05/19/09
 * This code licensed under the terms of the GPL.  See doc/LICENSE.
 */
public class ChannelPane extends JPanel implements IChannelPane, IChannelListener{
	private static final long serialVersionUID = 1L;
	private JScrollPane messagesWrap;
	private JTextArea messages;
	protected JTextArea prompt;
	private JPopupMenu messagesMenu;
	private JList userList;
	private JTextArea topicBar;
	protected final Channel channel;
	
	public ChannelPane(final Channel channel){
		setLayout(new BorderLayout());
		
		messages = new JTextArea();
		messages.setEditable(false);
		messages.setLineWrap(true);
		messages.addMouseListener(new MouseAdapter(){
			public void mouseReleased(MouseEvent e){
				if (e.isPopupTrigger()){
					showPopup(e);
				}
			}
		});
		
		messagesMenu = new JPopupMenu();
		JMenuItem clear = new JMenuItem(Messages.get("Clear"));
		clear.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				messages.setText("");
			}
		});
		messagesMenu.add(clear);
		
		messagesWrap = new JScrollPane();
		messagesWrap.getViewport().add(messages);
		
		add(BorderLayout.CENTER, messagesWrap);
		
		prompt = createPrompt();
		add(BorderLayout.SOUTH, prompt);
		
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				prompt.requestFocusInWindow();
			}
		});
		
		this.channel = channel;
		setName(channel.name);
		channel.addListener(this);
		
		JScrollPane scroller = new JScrollPane();
		userList = new JList();
		userList.setFixedCellWidth(110);
		scroller.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		scroller.getViewport().add(userList);
		
		add(BorderLayout.EAST, scroller);
		topicBar = new JTextArea();
		topicBar.setDragEnabled(true);
		topicBar.setEditable(true);
		topicBar.setLineWrap(true);
		topicBar.setWrapStyleWord(true);
		prompt.addKeyListener(new KeyAdapter(){
			public void keyPressed(KeyEvent e){
				/*if(e.getSource() instanceof java.awt.Component
					&& ((Component)e.getSource()).getName().equals("prompt")){}*/
				JTextArea prompt = (JTextArea) e.getSource();
				
				if(e.getKeyCode() == KeyEvent.VK_ENTER){
					CommandParser.Command cmd = CommandParser.parse(prompt.getText());
					switch(cmd){
					case EMPTY:
						break;
					case LIST_CHANNEL:
						//FIXME do something
						break;
					case JOIN_CHANNEL:
						//FIXME do something
						break;
					case PART_CHANNEL:
						channel.part();
						break;
					case CHAT:
						channel.sendMessage(prompt.getText());
						break;
					}
						
					prompt.setText("");
					prompt.setRows(0);
					e.consume();
				}
			}
		});
		add(BorderLayout.NORTH, topicBar);
	}
	
	@Override
	public void setUserList(List<AdvUser> users){
		UserListModel m = new UserListModel(users);
		userList.setModel(m);
	}
	
	private UserListModel getUserlistModel(){
		return (UserListModel) userList.getModel();
	}
	
	@Override
	public void removeUser(String user){
		UserListModel m = getUserlistModel();
		if(hasUser(user))
		{
			m.remove(m.hasElement(user));//TODO 좀더 직관적인 함수 구현의 필요성
		}
	}
	
	@Override
	public boolean hasUser(String user){
		UserListModel m = getUserlistModel();
		int uidx = m.hasElement(user);
		return uidx != -1;
	}
	
	@Override
	public void setTopic(String topic){
		topicBar.setText(topic);
	}
	
	private JTextArea createPrompt(){
		JTextArea c = new JTextArea();
		c.setName("prompt");
		c.setDragEnabled(true);
		c.setEditable(true);
		c.setLineWrap(true);
		c.setWrapStyleWord(true);
		return c;
	}
	
	@Override
	public void println(String s){
		print(s + "\n");
	}
    
	@Override
	public void print(String s){
		Document doc = messages.getDocument();
		try{
			//String time = new java.util.Date().toString();
			doc.insertString(doc.getLength(),s,null);
			printAfter();
		}catch(javax.swing.text.BadLocationException e){
			e.printStackTrace();
		}
	}
    
	//글줄에서 사용자가 보는 화면을 맨 아래로 옮김
	private void printAfter(){
		Runnable doAppend = new Runnable() {
			public void run() {
				JScrollBar sb = messagesWrap.getVerticalScrollBar();
				sb.setValue(sb.getMaximum());
			}
		};
		SwingUtilities.invokeLater(doAppend);
	}

	private void showPopup(MouseEvent e){
		messagesMenu.show(e.getComponent(),e.getX(),e.getY());
	}

	@Override
	public boolean hasChannel(Channel channel) {
		return channel.equals(channel);
	}

	@Override
	public void userListChanged() {
		setUserList(channel.getUsers());
	}
	
	@Override
	public void userAdded(AdvUser user){
		getUserlistModel().addElement(user);
	}

	@Override
	public void messageRecieved(String message) {
		println(message);
	}

	@Override
	public void userRemoved(AdvUser user, String message) {
		removeUser(user.getNick());
		println(message);
	}

	@Override
	public void topicChanged(String topic) {
		setTopic(topic);
	}

	@Override
	public void messageSended(String message) {
		// TODO Auto-generated method stub
		
	}
}