package net.kldp.zeta.wherithlov.ui;

import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import net.kldp.zeta.commons.resource.Messages;
import net.kldp.zeta.wherithlov.data.Player;
import net.kldp.zeta.wherithlov.data.Server;

import org.apache.commons.lang.StringUtils;

/**
 * @author ni2 (netisinfinite@gmail.com)
 * @version 0.1 11/25/06
 * This code licensed under the terms of the GPL.  See doc/LICENSE.
 */
public class ConnectionDialog extends JDialog{
	private static final long serialVersionUID = 1L;
	private JTextField txtNick;
	private JTextField txtLogin;
	private JTextField txtServerHost;
	private JTextField txtServerPort;
	private JTextField txtChannels;
	private JTextField txtEncoding;
	private JButton btnOk;
	private JButton btnCancel;
	
	public static final String COMMAND_CONNECT = "OK";
	public static final String COMMAND_CANCEL = "CANCEL";
	
	public ConnectionDialog(Frame owner){
		super(owner, Messages.get("Setup"), true);
		JPanel panel = new JPanel();
		add(panel, BorderLayout.CENTER);
		panel.setLayout(new GridLayout(0,2));
		panel.add(new JLabel(Messages.get("Nick")));
		txtNick = new JTextField();
		panel.add(txtNick);
		panel.add(new JLabel(Messages.get("Login")));
		txtLogin = new JTextField();
		panel.add(txtLogin);
		panel.add(new JLabel(Messages.get("ServerHost")));
		txtServerHost = new JTextField("irc.hanirc.org");
		panel.add(txtServerHost);
		panel.add(new JLabel(Messages.get("ServerPort")));
		txtServerPort = new JTextField("6667");
		panel.add(txtServerPort);
		panel.add(new JLabel(Messages.get("Channels")));
		txtChannels = new JTextField("#gnome"); //콤마로 구분
		panel.add(txtChannels);
		panel.add(new JLabel("Encoding"));
		txtEncoding = new JTextField("EUC-KR");
		panel.add(txtEncoding);
		final ConnectionDialog dialog = this;
		ActionListener disabler = new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				dialog.setVisible(false);
			}
		};
		btnOk = new JButton(Messages.get("OK"));
		btnOk.addActionListener(disabler);
		btnOk.setActionCommand(COMMAND_CONNECT);
		btnCancel = new JButton(Messages.get("Cancel"));
		btnCancel.addActionListener(disabler);
		btnCancel.setActionCommand(COMMAND_CANCEL);
		panel.add(btnOk);
		panel.add(btnCancel);
		pack();
	}
	
	public void addActionListener(ActionListener listener){
		btnOk.addActionListener(listener);
		btnCancel.addActionListener(listener);
	}
	
	public String getNick(){
		return txtNick.getText();
	}
	
	public String getLogin(){
		return txtLogin.getText();
	}
	
	public String getServerHost(){
		return txtServerHost.getText();
	}
	
	public int getServerPort(){
		return Integer.parseInt(txtServerPort.getText());
	}
	
	public String[] getChannels(){
		return StringUtils.split(txtChannels.getText(),",");
	}
	
	public String getEncoding(){
		return txtEncoding.getText();
	}
	
	public Player getPlayer(){
		return new Player(
				getNick(), 
				getLogin()
				); 
	}
	
	public Server getServer(){
		return new Server(
				getServerHost(), 
				getServerPort(), 
				getEncoding()
				);
	}
	
	public boolean isFilled(){
		Player player = getPlayer(); 
		Server server = getServer();
		String[] channels = getChannels();
		
		return (player != null && player.isFilled()) && (server != null && server
				.isFilled()) && channels.length > 0 ? true : false;
	}
}
