package net.kldp.zeta.wherithlov.ui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTabbedPane;

import net.kldp.zeta.commons.resource.Messages;
import net.kldp.zeta.wherithlov.data.Channel;
import net.kldp.zeta.wherithlov.data.IChannelCollectionListener;
import net.kldp.zeta.wherithlov.data.IConnectionListener;
import net.kldp.zeta.wherithlov.manager.IrcClient;
import net.kldp.zeta.wherithlov.ui.action.CloseTabAction;

public class MainPane extends JPanel implements IChannelCollectionListener, IConnectionListener, WindowListener{
	private static final long serialVersionUID = 1L;
	private final JTabbedPane pane;
	private IrcClient client;

	private JPopupMenu tabPopupMenu;
	//Logging received messages
	private IMessagePane messagePane;
	private final List<IChannelPane> channelPanes = new ArrayList<IChannelPane>();

	public MainPane(){
		setLayout(new BorderLayout());
		
		pane = new JTabbedPane();
		pane.addMouseListener(new MouseAdapter(){
			public void mouseReleased(MouseEvent e){
				if(e.isPopupTrigger()) showPopup(e.getComponent(),e);
			}
		});
		
		tabPopupMenu = new JPopupMenu();
		JMenuItem closeMenuItem = new JMenuItem(Messages.get("Close"));
		closeMenuItem.addActionListener(new CloseTabAction(this));
		tabPopupMenu.add(closeMenuItem);
		
		messagePane = new MessagePane();
		pane.addTab(
				"Messages", 
				null, 
				(JPanel) messagePane, 
				"Messages");
		
		add(BorderLayout.CENTER, pane);
	}
	
	/**
	 * ConnectionDialog와 IrcClient를 연결한다.
	 * @param dialog
	 */
	public void connect(final ConnectionDialog dialog){
		client = new IrcClient(this);
		dialog.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if(e.getActionCommand() == ConnectionDialog.COMMAND_CONNECT && 
						dialog.isFilled()){
					client.connect(
							dialog.getPlayer(),
							dialog.getServer(),
							dialog.getChannels());
				}
			}
		});
	}
	
	/**
	 * MessagePane에 정보 메시지를 출력한다.
	 */
	private void printNotice(String message){
		messagePane.println(message);
	}
	
	/**
	 * 채널 화면에 메시지를 출력한다.
	 * @param channel 메시지를 출력할 채널
	 * @param message 출력할 메시지
	 */
	public void printMessage(Channel channel, String message){
		getChannelPane(channel).println(message);
	}
	
	public Component createChannelPane(Channel channel){
		ChannelPane c = new ChannelPane(channel);
		channelPanes.add(c);
		pane.add(c);
		pane.setSelectedComponent(c);
		//setToolTipTextAt(0, "Messages");
		
		return c;
	}
	
	//TODO 존재하지 않는 채널일 경우의 예외 처리
	public IChannelPane getChannelPane(Channel channel){
		return (IChannelPane) pane.getComponentAt(pane.indexOfTab(channel.name));
	}
	
	public void destroyChannelPane(Channel channel){
		for(IChannelPane c : channelPanes){
			if(c.hasChannel(channel)){
				channelPanes.remove(c);
				pane.remove((Component)c);
				break;
			}
		}
	}

	private int eventX, eventY;
	
	private void showPopup(Component destComp, MouseEvent e){
		if(pane.indexAtLocation(e.getX(),e.getY())>=0){
			eventX=e.getX();
			eventY=e.getY();
			
			tabPopupMenu.show(destComp,eventX,eventY);
		}
	}
	
	public void requestCloseTab(){
		//이벤트가 발생한 위치로부터 해당되는 패널의 인덱스를 가져온다.
		closeTab(pane.indexAtLocation(eventX,eventY));
	}
	
	/**
	 * 인덱스에 해당하는 패널을 닫는다.
	 * @param paneIdx
	 */
	private void closeTab(int paneIdx){
		Component comp = pane.getComponentAt(paneIdx);
		//채널의 경우는 바로 닫지 않고 서버에서 닫혔다는 응답이 오면 IrcClient에서 닫는다.
		if(comp instanceof ChannelPane)
			((ChannelPane) comp).channel.part();
	}

	@Override
	public void channelAdded(Channel channel) {
		createChannelPane(channel);
	}

	@Override
	public void channelCollectionChanged() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void channelRemoved(Channel channel) {
		destroyChannelPane(channel);
	}

	@Override
	public void connectionMessageRecieved(String message) {
		printNotice(message);
	}

	@Override
	public void windowActivated(WindowEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowClosed(WindowEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowClosing(WindowEvent e) {
		client.disconnectAll();
	}

	@Override
	public void windowDeactivated(WindowEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowDeiconified(WindowEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowIconified(WindowEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowOpened(WindowEvent e) {
		// TODO Auto-generated method stub
		
	}
}
