package net.kldp.zeta.wherithlov.ui;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.text.Document;

import net.kldp.zeta.commons.resource.Messages;

/**
 * 서버 메시지 및 공지사항 등을 출력하는 패널.
 * @author ni2 (netisinfinite@gmail.com)
 * @version 0.2 05/19/09
 * This code licensed under the terms of the GPL.  See doc/LICENSE.
 */
public class MessagePane extends JPanel implements IMessagePane{
	private static final long serialVersionUID = 1L;
	private JScrollPane messagesWrap;
	private JTextArea messages;
	protected JTextArea prompt;
	private JPopupMenu messagesMenu;
	
	public MessagePane(){
		setLayout(new BorderLayout());
		
		messages = new JTextArea();
		messages.setEditable(false);
		messages.setLineWrap(true);
		//c.setFont(new Font("monospaced", Font.PLAIN, 12));
		messages.addMouseListener(new MouseAdapter(){
			public void mouseReleased(MouseEvent e){
				if (e.isPopupTrigger()){
					showPopup(e);
				}
			}
		});
		
		messagesMenu = new JPopupMenu();
		JMenuItem clear = new JMenuItem(Messages.get("Clear"));
		clear.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				messages.setText("");
			}
		});
		messagesMenu.add(clear);
		
		messagesWrap = new JScrollPane();
		messagesWrap.getViewport().add(messages);
		
		add("Center", messagesWrap);
		
		prompt = createPrompt();
		add("South", prompt);
		
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				prompt.requestFocusInWindow();
			}
		});
	}
	
	private JTextArea createPrompt(){
		JTextArea c = new JTextArea();
		c.setName("prompt");
		c.setDragEnabled(true);
		c.setEditable(true);
		c.setLineWrap(true);
		c.setWrapStyleWord(true);
		return c;
	}
	
	@Override
	public void println(String s){
		print(s + "\n");
	}
    
	@Override
	public void print(String s){
		Document doc = messages.getDocument();
		try{
			//String time = new java.util.Date().toString();
			doc.insertString(doc.getLength(),s,null);
			printAfter();
		}catch(javax.swing.text.BadLocationException e){
			e.printStackTrace();
		}
	}
    
	//글줄에서 사용자가 보는 화면을 맨 아래로 옮김
	private void printAfter(){
		Runnable doAppend = new Runnable() {
			public void run() {
				JScrollBar sb = messagesWrap.getVerticalScrollBar();
				sb.setValue(sb.getMaximum());
			}
		};
		SwingUtilities.invokeLater(doAppend);
	}

	private void showPopup(MouseEvent e){
		messagesMenu.show(e.getComponent(),e.getX(),e.getY());
	}
}
