//http://dev.eclipse.org/newslists/news.eclipse.platform.swt/msg09787.html

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.graphics.Region;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

public class GoogleShell {
	public static void main(String[] args) throws MalformedURLException, IOException {
		final Display display= new Display();
		final Shell shell= GoogleShell.create(
			display,
			new ImageData(new URL("http://www.google.com/images/logo.gif").openStream()));
		shell.open();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		display.dispose();
	}

	public static Shell create(
		final Display display,
		ImageData imagedata)
		throws MalformedURLException, IOException {
		
		final Shell shell= new Shell(display, SWT.NO_TRIM | SWT.ON_TOP);
		final Canvas canvas= new Canvas(shell, SWT.NONE);
		final int style = Style.ON_TOP | Style.MOVEABLE;

		imagedata.transparentPixel= imagedata.getPixel(0, imagedata.height - 1);
		final Image image= new Image(display, imagedata, imagedata.getTransparencyMask());

		// define a region that looks like the non-transparent part of the
		// image...
		Region region= new Region();
		Rectangle bounds= image.getBounds();
		region.add(bounds);

		if ((style & Style.CLOSE_BUTTON) != 0) {
			region.subtract(
			new Rectangle(0, 0, imagedata.width - 4, 0));
		}else {
			region.subtract(new Rectangle(0, 0, imagedata.width, 0));
		}

		for (int y= 0; y < imagedata.height; y++) {
			for (int x= 0; x < imagedata.width; x++) {
				if (imagedata.transparentPixel == imagedata.getPixel(x, y)) {
					region.subtract(new Rectangle(x, y, 1, 1));
				}
			}
		}


		//define the shape of the shell using setRegion
		shell.setRegion(region);
		Rectangle size= region.getBounds();
		shell.setSize(size.width, size.height);
		FormLayout formLayout= new FormLayout();
		shell.setLayout(formLayout);
		FormData formData= new FormData();
		formData.top= new FormAttachment(0, 0);
		formData.left= new FormAttachment(0, 0);
		formData.right= new FormAttachment(100, 0);
		formData.bottom= new FormAttachment(100, 0);
		canvas.setLayoutData(formData);

		//add ability to move shell around
		Listener l= new Listener() {
			Point origin;
			public void handleEvent(Event e) {
				switch (e.type) {
					case SWT.MouseDown :
					origin= new Point(e.x, e.y);
					break;
					case SWT.MouseUp :
					origin= null;
					break;
					case SWT.MouseMove :
					if (origin != null && ((style & Style.MOVEABLE) != 0)) {
						Point p= display.map(shell, null, e.x, e.y);
						shell.setLocation(p.x - origin.x, p.y - origin.y);
					}
					break;
					case SWT.MouseDoubleClick :
					break;
				}
			}
			};
			canvas.addListener(SWT.MouseDown, l);
			canvas.addListener(SWT.MouseUp, l);
			canvas.addListener(SWT.MouseMove, l);
			canvas.addListener(SWT.MouseDoubleClick, l);
			canvas.addPaintListener(new PaintListener() {
			public void paintControl(PaintEvent e) {
				e.gc.drawImage(image, 0, 0);
			}
		});
		return shell;
	}

	private static class Style {
		public static final int NONE= 0;
		public static final int DEFAULT= -1;
		public static final int ON_TOP= 1 << 1;
		public static final int CLOSE_BUTTON= 1 << 2;
		public static final int MOVEABLE= 1 << 3;
	}


}
