package org.kldp.zeta;

import org.kldp.zeta.interpret4j.Interpreter;
import org.kldp.zeta.interpret4j.Interpreter.Type;

/**
 * @author ni2
 * @version 0.1, 09/01/06
 * This code licensed under the terms of the GPL.  See doc/LICENSE.
 *
 * bsh, xml, js, jy 파일을 다루는 인터프리터 컨트롤러.
 */
public class ZetaInterpreter{
	private static final ZetaInterpreter controller = new ZetaInterpreter();
	private final Interpreter interpreter;
	
	public static ZetaInterpreter instance(){
		return controller;
	}
	
	private ZetaInterpreter(){
		interpreter = Interpreter.instance();
		
		try{
			//초기화 스크립트를 실행한다.
			interpreter.interpretFromFile(Zeta.BSH_IMPORT_PATH, Type.BEANSHELL);
			
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	
	/**
	 * 실행할 스크립트를 입력받아 적절한 함수로 넘긴다.
	 */
	public void interpret(String fileName){
		Object result = null;
		try {
			result = interpreter.interpretFromFile(fileName);
		} catch (Exception e) {
			Zeta.logger.printStackTrace(e);
		}
		
		if(result != null)
			Zeta.openWidget(new ZetaModule(fileName, result));
	}
	
	
	/**
	 * 입력받은 bsh 명령을 실행한다.
	 */
	public void bshInterpret(String command){

		/*
		//스크립트 내에서 addClassPath 명령시 디렉터리 지정할 필요가 없도록
		//bsh 인터프리터의 패스를 스크립팅 파일이 있는 위치로 옮겨 준다.
		String fileDirectory = new java.io.File(fileName).getDirectory();
		bsh.eval("cd(" + fileDirectory + ")");
		result = bsh.source(fileName);
		//분석이 끝나면 다시 복구한다.
		bsh.eval("cd(" + startDirectory + ")");
		*/
		
		try{
			Object result = interpreter.eval(command, Type.BEANSHELL);
			if(result != null) Zeta.logger.println(result.toString());
			
		}catch(Exception e){
			Zeta.logger.printStackTrace(e);
		}
	}
}
